MewBufferBinding

Trait MewBufferBinding 

pub trait MewBufferBinding: MewBuffer {
    // Required methods
    fn binding_type() -> BindingType;
    fn as_binding<'a>(&'a self) -> BindingResource<'a>;
}
Trait for internal use to work specifically with buffers that can be put in a bind group.

Required Methods

fn binding_type() -> BindingType

Get a wgpu::BindingType to use for this type of buffer.

fn as_binding<'a>(&'a self) -> BindingResource<'a>

Get this buffer’s wgpu::BindingResource to bind it in a bind group.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors