pub struct ExampleBindGroup {
pub sampler: Sampler,
pub texture: Texture,
pub buffer: ShaderBuffer,
/* private fields */
}Fields
sampler: Samplertexture: Texturebuffer: ShaderBufferTrait Implementations
Sourceimpl Clone for ExampleBindGroup
impl Clone for ExampleBindGroup
Sourcefn clone(&self) -> ExampleBindGroup
fn clone(&self) -> ExampleBindGroup
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Debug for ExampleBindGroup
impl Debug for ExampleBindGroup
Sourceimpl Deref for ExampleBindGroup
impl Deref for ExampleBindGroup
Sourceimpl Hash for ExampleBindGroup
impl Hash for ExampleBindGroup
Sourceimpl MewBindGroup for ExampleBindGroup
impl MewBindGroup for ExampleBindGroup
Sourcetype BufferSet = (Sampler, Texture, ShaderBuffer)
type BufferSet = (Sampler, Texture, ShaderBuffer)
A tuple of all the bind group’s internal buffers.
Sourcetype BufferArray<T> = [T; 3]
type BufferArray<T> = [T; 3]
A generic array the length of the number of buffers (literally
[T; N]).Sourcefn new(bind_group: BindGroup, buffers: Self::BufferSet) -> Self
fn new(bind_group: BindGroup, buffers: Self::BufferSet) -> Self
Build this bind group type from a
wgpu::BindGroup
and all its internal buffers in a tuple. Read moreSourcefn layout_entries() -> &'static Self::BufferArray<BindGroupLayoutEntry>
fn layout_entries() -> &'static Self::BufferArray<BindGroupLayoutEntry>
Get an array of
wgpu::BindGroupLayoutEntrys
for all internal buffers.Sourcefn layout_desc() -> BindGroupLayoutDescriptor<'static>
fn layout_desc() -> BindGroupLayoutDescriptor<'static>
Get a
wgpu::BindGroupLayoutDescriptor
to build this type of bind group.Sourcefn bind_group_entries(
buffers: &Self::BufferSet,
) -> Self::BufferArray<BindGroupEntry<'_>>
fn bind_group_entries( buffers: &Self::BufferSet, ) -> Self::BufferArray<BindGroupEntry<'_>>
Sourcefn bind_group_desc<'a>(
layout: &'a MewBindGroupLayout<Self>,
entries: &'a Self::BufferArray<BindGroupEntry<'_>>,
) -> BindGroupDescriptor<'a>
fn bind_group_desc<'a>( layout: &'a MewBindGroupLayout<Self>, entries: &'a Self::BufferArray<BindGroupEntry<'_>>, ) -> BindGroupDescriptor<'a>
Get a
wgpu::BindGroupDescriptor
to build this bind group, using its layout and a reference to its
wgpu::BindGroupEntrys
from Self::bind_group_entries.Sourceimpl Ord for ExampleBindGroup
impl Ord for ExampleBindGroup
Sourcefn cmp(&self, other: &ExampleBindGroup) -> Ordering
fn cmp(&self, other: &ExampleBindGroup) -> Ordering
1.21.0 · Sourcefn max(self, other: Self) -> Selfwhere
Self: Sized,
fn max(self, other: Self) -> Selfwhere
Self: Sized,
Compares and returns the maximum of two values. Read more
Sourceimpl PartialEq for ExampleBindGroup
impl PartialEq for ExampleBindGroup
Sourceimpl PartialOrd for ExampleBindGroup
impl PartialOrd for ExampleBindGroup
impl Eq for ExampleBindGroup
impl StructuralPartialEq for ExampleBindGroup