RenderContextBuilder

Struct RenderContextBuilder 

pub struct RenderContextBuilder { /* private fields */ }
A builder abstraction for making a RenderContext.

It’s possible to make one directly with RenderContext::new, but you need to provide everything manually. This builder takes care of initilizing the wgpu handles for you with default values.

Implementations

Source

impl RenderContextBuilder

pub fn new_with_instance(instance: Instance) -> Self

Prepare a new builder, providing your own wgpu::Instance.

pub fn new_with_backends(backends: Backends) -> Self

Prepare a new builder, specifying the wgpu::Backends you want to support.

The wgpu::Instance will be built later when needed with these backends.

pub fn with_compatible_surface( &mut self, surface: Option<Arc<Surface<'static>>>, ) -> &mut Self

Specify a compatible surface for the wgpu::Adapter.

This is required by WebGL and really annoying to set up, see winitutils::DeferredContext.

pub fn with_features(&mut self, features: Features) -> &mut Self

Specify the wgpu::Features the wgpu::Device will require.

pub fn with_limits(&mut self, limits: Limits) -> &mut Self

Specify the wgpu::Limits the wgpu::Device will require.

On web, this is automatically set to wgpu::Limits::downlevel_webgl2_defaults, but is overridable if you really want to.
On native, it’s just Default::default.

pub fn get_instance(&mut self) -> Instance

Get a copy of the wgpu::Instance that’ll be used to get the rest of the wgpu handles.

If this wasn’t provided when building, it’ll be generated now.

pub async fn build(self) -> Result<RenderContext, BuildContextError>

Try to build the RenderContext with the configured settings.

This function is async because wgpu::Instance::request_adapter and wgpu::Adapter::request_device are for some reason.
On native, you can use Self::build_poll. On web, you need to spawn a JS thread to await this, or use Self::deferred which does this internally.

pub fn build_poll(self) -> Result<RenderContext, BuildContextError>

Available on crate features poll only.

Try to build the RenderContext with the configured settings, but block on the result with pollster.

pub fn deferred(self) -> DeferredContext

Available on crate feature winit and (crate features webgl or poll) only.

Put this builder into a winitutils::DeferredContext, which on web will automate running Self::with_compatible_surface when a window is created and build the RenderContext as soon as it can.