MewVertexStructByteSlice

Trait MewVertexStructByteSlice 

pub trait MewVertexStructByteSlice {
    // Required method
    fn as_byte_slice(&self) -> &[u8] ;
}
Convenience trait to convert vertex struct slices directly to byte slices.

Automatically implemented on &[S] where S is any vertex struct.

Required Methods

fn as_byte_slice(&self) -> &[u8]

Get a byte slice to write directly to a buffer.

Implementations on Foreign Types

Source

impl<S: MewVertexStruct> MewVertexStructByteSlice for [S]

Source

fn as_byte_slice(&self) -> &[u8]

Implementors