pub enum ConfigSurfaceError {
CreateSurface(CreateSurfaceError),
InUse,
NotInitialized,
SurfaceError(SurfaceError),
Unsupported,
}Available on crate features
winit only.Expand description
Error returned when configuring a surface or making a texture view for the surface.
Variants
CreateSurface(CreateSurfaceError)
wgpu could not create a surface, see
wgpu::Instance::create_surface.
InUse
The surface already has an active view - a new one can’t be created, nor can the current one be reconfigured.
NotInitialized
Can’t reconfigure the surface because it hasn’t been created or configured
yet. Non-fatal, but run SurfaceWindow::init_with_context first.
SurfaceError(SurfaceError)
Trying to use the surface failed. Non-fatal, but call
SurfaceWindow::drop_surface to re-initialize it. Also see
wgpu::SurfaceError.
Unsupported
The surface that tried to be configure wasn’t supported by this adapter. Don’t
mix & match resources between RenderContexts!