SurfaceWindow

Struct SurfaceWindow 

pub struct SurfaceWindow { /* private fields */ }
Available on crate features winit only.
Implementations

Source

impl SurfaceWindow

pub fn new( event_loop: &ActiveEventLoop, attributes: WindowAttributes, config_opts: SurfaceConfigOptions, ) -> Result<Self, OsError>

Create a new window under an winit::ActiveEventLoop.

Also requires a winit::window::WindowAttributes (though Default should work fine for most cases) and a SurfaceConfigOptions (Default should be fine here too).

pub fn try_get_config(&self) -> Option<SurfaceConfiguration>

Try to get the wgpu::SurfaceConfiguration, if the surface has been created.

pub fn window(&self) -> &Window

Get a reference to the window.

pub fn init_surface( &self, instance: &Instance, adapter: &Adapter, device: &Device, ) -> Result<ActiveSurfaceWindow<'_>, ConfigSurfaceError>

Opaquely create, configure, and get the wgpu::SurfaceTexture & wgpu::TextureView of the underlying winit::window::Window.

If the window’s wgpu::Surface hasn’t been created yet, it will do so now. If needed it’ll make a new wgpu::SurfaceConfiguration using either the window’s inner size or a manually provided one from the last Self::resize.

Next, it’ll try getting its wgpu::SurfaceTexture with wgpu::Surface::get_current_texture. This may fail if the surface was old and needed to be refreshed, in which case it’ll need to fully recreate the surface. If getting the surface texture fails a 2nd time, the error will be raised to the caller.

Only one ActiveSurfaceWindow is allowed at any one time, this is enforced during runtime.

pub fn init_with_context( &self, context: &RenderContext, ) -> Result<ActiveSurfaceWindow<'_>, ConfigSurfaceError>

Get a usable ActiveSurfaceWindow, which guarantees that the window surface exists & is configured.

See Self::init_surface for more details.

pub fn try_drop_surface(&self) -> Result<(), BorrowMutError>

Try to drop the surface.

pub fn drop_surface(&mut self)

Drop the surface now.

pub fn resize(&self, size: Option<PhysicalSize<u32>>)

Set the target window size and trigger a reconfiguration.

If you pass None, the size will be queried from the window, or you can specify it from a winit resize event. The surface will be reconfigured when it’s next initialized.

pub fn reconfigure(&self)

Mark the surface to be reconfigured when it’s next initialized.

pub fn get_size(&self) -> PhysicalSize<u32>

Convenience function to get the window’s inner size.