pub trait MewVertexStructByteSlice {
// Required method
fn as_byte_slice(&self) -> &[u8] ⓘ;
}Expand description
Convenience trait to convert vertex struct slices directly to byte slices.
Automatically implemented on &[S] where S is any vertex struct.
Required Methods
Sourcefn as_byte_slice(&self) -> &[u8] ⓘ
fn as_byte_slice(&self) -> &[u8] ⓘ
Get a byte slice to write directly to a buffer.